
get_pval_stars <- function(x) {
  symnum(x, corr = FALSE, na = FALSE, 
         cutpoints = c(0, 0.001, 0.01, 0.05, 1),
         symbols = c("***", "**", "*", ""))
}


update_string_form <- function(base_form, add_form = "", 
                               remove_form = "", 
                               add_fe = "") {
  base_form_char <- base_form |> as.character()
  str_c(
    base_form_char[[2]], " ~ ", 
    base_form_char[[3]], 
    if_else(add_form != "", str_c(" + " , add_form), ""),
    if_else(remove_form != "", 
            str_c(" - " , str_replace_all(remove_form, "\\+", "-")), 
            ""),
    if_else(add_fe != "", str_c(" | " , add_fe), "")
  ) |> 
    as.formula()
}

sum_models <- function(mods) {
  
  modelsummary(
    models = mods,
    estimate = "{estimate}{stars}",
    # statistic = NULL,
    statistic = "({std.error})",
    stars = c("*" = 0.1, "**" = 0.05, "***" = 0.01),
    coef_rename = c(
      "poly(area_ha..log, 2)1" = "Area HA (log)",
      "poly(area_ha..log, 2)2" = "Area HA (log squared)",
      "war_dum" = "War dummy (0|1)",
      
      "BalRil..log" = "Bonitat (log)",
      "BalRil..revdum" = "Missing bonitat rev. dummy (0|1)",
      
      "price_nmv_missing" = "Missing NMV (0|1)",
      "price_below_nmv" = "Sales below NMV (0|1)",
      "price_as_nmv" = "Sales as NMV (0|1)",
      "purpose_descr_short_301.07-19 Other" = "01.07-19 Other",
      "legalent" = "Legal entity, (0|1)",
      "subjects_n" = "N benef.",
      "transaction_n" = "N sales",
      "transaction_order" = "Repeated (1-last/5-old)",
      "mean_elev..log" = "Elevation, m (log)",
      "mean_slope..log" = "Slope, degree (log)",
      
      "dist_elevator..log" = "Grain storage, km (log)",
      "dist_motorwayPro..log" = "Motorway, km (log)",
      "dist_primaryPro..log" = "Primary road, km (log)",
      "dist_secondaryroad..log" = "Secondary road, km (log)",
      "dist_tertiaryroad..log" = "Tertiary road, km (log)",
      "dist_sett..log" = "Settlement, km (log)",
      "dist_city..log" = "City, km (log)",
      "dist_railway..log" = "Railway, km (log)",
      "dist_portPro..log" = "Port terminal, km (log)",
      
      
      "gtk_91..log" = "Hydro-thermic coef. 1991-2023, (log)",
      "gtk_61..log" = "Hydro-thermic coef. 1961-1990, (log)",
      
      "dist_lake..log" = "Lake shallow, km (log)",
      "dist_reservoir..log" = "Reservoit, km (log)",
      "dist_river_narrow..log" = "River narrow, km (log)",
      "dist_river_wide..log" = "River wide, km (log)",
      
      "shbuiltareaL1" = "Builtup area share (L1)",
      "shcultareaL1" = "Crop area share (L1)",
      "shforsareaL1" = "Forest area share (L1)",
      "shgrasareaL1" = "Grassland area share (L1)",
      "shwatrareaL1" = "Water area share (L1)",
      
      "shbuiltareaL2" = "Builtup area share (L2)",
      "shcultareaL2" = "Crop area share (L2)",
      "shforsareaL2" = "Forest area share (L2)",
      "shgrasareaL2" = "Grassland area share (L2)",
      "shwatrareaL2" = "Water area share (L2)",
      "year2022" = "Year 2022 (0|1)",
      "year2023" = "Year 2023 (0|1)",
      "year2024" = "Year 2024 (0|1)" ,
      "soil_density..log" = "Soil density, cg/cm3 (log)",
      "soil_nitrogen..log" = "Soil nitrogen, cg/kg (log)",
      "poly(soil_nitrogen..log, 2)1" = "Soil nitrogen, cg/kg (log)",
      "poly(soil_nitrogen..log, 2)2" = "Soil nitrogen, cg/kg (log squared)",
      "soil_phh2o..log" = "Soil pH, log" ,
      "poly(soil_phh2o..log, 2)1" = "Soil pH (log)" ,
      "poly(soil_phh2o..log, 2)2" = "Soil pH (log squared)" ,
      "soil_organic_content..log" = "Soil carbon, t/ha (log)",
      "poly(soil_orgcont..log, 2)1" = "Soil carbon, t/ha (log)",
      "poly(soil_orgcont..log, 2)2" = "Soil carbon, t/ha (log squared)" 
    ),
    coef_omit = "(Intercept)",
    shape = term ~ model,
    gof_map = c("nobs", "adj.r.squared", "r2.within.adjusted", "rmse"),
    output = "flextable"
  ) |> 
    add_footer_lines(
      value = as_paragraph(c(
        "Significance levels are: `*` < 0.1, '**' < 0.05, and '***' < 0.01",
        "Robust standard errors clustered at the level of fixed effects are reported in brackets."
      ))
    ) 
}


FitFlextableToPage <- function(ft, pgwidth = 6.5) {
  ft_out <- 
    ft |> fontsize(size = 9, part = "all") |> autofit()
  
  ft_out <- width(ft_out,
                  width = dim(ft_out)$widths * pgwidth / (flextable_dim(ft_out)$widths))
  return(ft_out)
}

get_mean_fit <- function(
    dta, 
    price_var = "lprice_uah_2024_usd_ha_win",
    groups_order = c("adm8_id", "adm7_id", "adm6_id", "adm3_id", "adm2old_id", "adm2_id", "adm1_id"), 
    ... ) {
  
  from_dta <- 
    dta |> 
    mutate(price_ha = .data[[price_var]]) |> 
    select(price_ha, any_of(groups_order)) 
  
  from_dta |> group_by(pick(any_of(groups_order))) |> list() |>
    append(as.list(groups_order)) |> 
    reduce(~{
      new_var = str_c(.y, "__price")
      .x |> 
        group_by(pick(all_of(.y))) |> 
        mutate({{new_var}} := ifelse(n() > 10, mean(price_ha), NA_real_))
    }) |> 
    ungroup() |> 
    mutate(price_pred = NA_real_) |> 
    list() |> 
    append(groups_order) |> 
    reduce(~{
      new_var <- str_c(.y, "__price")
      new_var <- sym(new_var)
      .x |>
        mutate(price_compare = !!new_var) |> 
        mutate(price_pred = ifelse(
        is.na(price_pred) &
          !is.na(price_compare),
        price_compare,
        price_pred
      )) |>
        select(-price_compare)
    })
}



predict_mean_fit <- function(test_dta,
                             fit_mean,
                             groups_order = c("adm8_id",
                                              "adm7_id",
                                              "adm6_id",
                                              "adm3_id",
                                              "adm2old_id",
                                              "adm2_id",
                                              "adm1_id"),
                             ...)
{
  fit_mean <- fit_mean |>
    select(price_pred , contains("adm")) |>
    distinct()
  
  
  all_test <-
    test_dta |>
    select(cadnum, lprice_uah_2024_usd_ha_win, contains("adm")) |>
    mutate(idd = row_number()) |>
    left_join(fit_mean, by = groups_order) |>
    select(idd, cadnum, lprice_uah_2024_usd_ha_win, everything()) |>
    select(-contains("__"))
  
  faild_comparison <-
    all_test |> filter(is.na(price_pred)) |>  list() |>
    append(groups_order[-1]) |>
    reduce( ~ {
      to_add <-
        fit_mean |> select(any_of(.y), any_of(str_c(.y, "__price"))) |>
        filter(if_all(contains("__price"), ~ !is.na(.))) |>
        distinct() |>
        group_by(pick(any_of(.y))) |>
        summarise(across(contains("__price"), ~ mean(., na.rm = TRUE))) |>
        ungroup() |>
        rename_with(.cols = any_of(str_c(.y, "__price")), .fn = ~ "new_price")
      .x |>
        left_join(to_add, by = .y) |>
        mutate(price_pred  = ifelse(is.na(price_pred), new_price, price_pred)) |>
        select(-new_price)
    })
  
  all_test |>
    filter(!is.na(price_pred)) |>
    bind_rows(faild_comparison) |>
    arrange(idd) |>
    select(-idd, -contains("adm")) |>
    rename(p_price = price_pred)
}


# 
# predict_mean_fit <- 
#   function(dta, 
#            price_var = "lprice_uah_2024_usd_ha_win", 
#            model_dta,
#            groups_order = c("adm8_id", "adm7_id", "adm6_id", "adm3_id", "adm2old_id", "adm2_id", "adm1_id"),
#            drop = TRUE,
#            ...) {
#   
#   # browser()
#     
#     out <- 
#       dta |>
#       mutate(price_actual = .data[[price_var]],
#              price_predicted = NA_real_,
#              price_predicted_type = NA_character_) |> 
#       select(price_actual, 
#              price_predicted, 
#              price_predicted_type, 
#              any_of(groups_order)
#              ) |>
#       list() |> append(as.list(groups_order)) |> 
#       reduce(~{
#         if (length(is.na(.x$price_predicted)) > 0) {
#           browser()
#           out1 <- 
#             .x |> 
#             left_join(
#               model_dta |> select(price_mean, any_of(.y)) |> distinct(),
#               by = .y
#             ) |> 
#             mutate(
#               price_predicted = 
#                 ifelse(is.na(price_predicted) & 
#                          !is.na(price_mean), price_mean, price_predicted)
#             ) |> 
#             select(-contains("price_mean"))
#         } else {
#           out1 <- .x
#         }
#         out1
#       })
#     
#     
#   # out <-
#   #   dta |>
#   #   mutate(price_actual = .data[[price_var]]) |> 
#   #   select(price_actual, any_of(groups_order)) |>  
#   #   left_join(
#   #     model_dta |> 
#   #       select(adm4_id, adm3_id, adm2_id, adm1_id, price_adm4) |> 
#   #       distinct(), 
#   #     by = join_by(adm4_id, adm3_id, adm2_id, adm1_id)) |>  
#   #   left_join(
#   #     model_dta |> select(adm3_id, adm2_id, adm1_id, price_adm3) |> distinct(), 
#   #     by = join_by(adm3_id, adm2_id, adm1_id)) |>  
#   #   left_join(
#   #     model_dta |> select(adm2_id, adm1_id, price_adm2) |> distinct(), 
#   #     by = join_by(adm2_id, adm1_id)) |> 
#   #   left_join(
#   #     model_dta |> select(adm1_id, price_adm1) |> distinct(), 
#   #     by = join_by(adm1_id)) |> 
#   #   mutate(
#   #     pred_price_ha = ifelse(!is.na(price_adm4), price_adm4, NA_real_),
#   #     pred_price_ha = ifelse(is.na(pred_price_ha), price_adm3, pred_price_ha),
#   #     pred_price_ha = ifelse(is.na(pred_price_ha), price_adm2, pred_price_ha),
#   #     pred_price_ha = ifelse(is.na(pred_price_ha), price_adm1, pred_price_ha)
#   #   ) 
#   if (drop) {
#     out <- out |> select(price_actual, price_predicted = pred_price_ha, ...)
#   }
#   out
# }

get_gofs <- function(dta, prefix = "", suffix = "") {
  dta |> 
    summarise(
      price_predicted_mean = mean(price_predicted, na.rm = TRUE),
      price_predicted_sd = sd(price_predicted, na.rm = TRUE),
      n_missing_pred = sum(is.na(price_predicted)),
      rmse = rmse_vec(price_actual, price_predicted),
      rsq = rsq_vec(price_actual, price_predicted),
      mae = mae_vec(price_actual, price_predicted)
    ) |> 
    rename_with(.fn = ~str_c(prefix, ., suffix), .cols = everything())
}

get_gofs_exp <- function(dta, prefix = "exp.", suffix = "") {
  dta |> mutate(across(contains("price"), ~ exp(.))) |>
    get_gofs() |>
    rename_with(.fn = ~ str_c(prefix, ., suffix), .cols = everything())
}

get_gofs_log <- function(dta, prefix = "log.", suffix = "") {
  dta |> mutate(across(contains("price"), ~ log(.))) |>
    get_gofs() |>
    rename_with(.fn = ~ str_c(prefix, ., suffix), .cols = everything())
}

get_gofs_for_logged <- function(dta) {
  dta |> get_gofs(prefix = "log.") |> 
    bind_cols(dta |> get_gofs_exp(prefix = "level.")) 
}

get_gofs_for_exp <- function(dta) {
  dta |> get_gofs(prefix = "level.") |> 
    bind_cols(dta |> get_gofs_log(prefix = "log.")) 
}

get_fef_predictions <- function(dta, model, price_var = "lprice_uah_2024_usd_ha_win") {
  dta |> 
    mutate(
      price_actual = .data[[price_var]],
      price_predicted = predict(model, newdata = dta)
    ) |> 
    select(price_actual, price_predicted)
}

# Fixed-effect relates functions ---------------------------

get_fefs <- 
  function(
    dta,
    model,
    prob = c(0, 0.01, 0.05, 0.1, 0.25, 0.5, 0.75, 0.90, 0.95, 0.99, 1)) {
    fe_dta <- predict(model, fixef = TRUE)
    fe_name <- names(fe_dta)
    fe_name_sym <- sym(fe_name)
    
    pred_fe_dta_0 <-
      dta |>
      mutate(fe = fe_dta[[1]] |> exp()) |>
      select(any_of(fe_name), fe) |>
      group_by(!!fe_name_sym, fe) |>
      summarise(n = n()) |>
      ungroup() |>
      rename(fe_id := !!fe_name_sym) 
    
    quant_th <- quantile(pred_fe_dta_0$fe, probs = prob)
    
    pred_fe_dta_0 |>
      mutate(quant = cut(fe, quant_th, labels = prob[-1], include.lowest = T)) 
    
    # list(pred_fe_dta_0) |> set_names(fe_name)
  }

# R2 contribution  ---------------------


get_contrib <- function(local_mods) {
  
  all_r2 <-
    drop_coef |>
    mutate(
      old_reg = 
        map(coef, ~ {
          rem_form <- .x
          local_mods |> imap_dfr( ~ {.x |> glance() |>
              mutate(model = .y, nvar = length(coef(.x)))})
        }),
      
      new_regs =
        map(coef, ~ {
          rem_form <- .x
          local_mods |>
            imap_dfr( ~ {
              fe_var <-
                formula(.x) |> as.character() |> str_extract("\\|+.{1,}$") |>
                str_replace_na("") |> str_remove("\\|") |> str_trim() |> 
                str_c(collapse = "")
              .x2 <- 
                update(.x, 
                       fml = update_string_form(base_form,
                                                remove_form = rem_form, 
                                                add_fe = fe_var)) 
              
              .x2 |> glance() |>
                mutate(model = .y, nvar = length(coef(.x2)))
            })
        })
    )
  
  all_r2 |> 
    unnest(old_reg, new_regs) |> 
    select(var, coef, model, contains("r.sq"),  contains("nvar")) |> 
    group_by(var) |> 
    mutate(
      within.r.squared = ifelse(is.na(within.r.squared) & row_number() == 1, adj.r.squared, within.r.squared),
      within.r.squared1 = ifelse(is.na(within.r.squared1) & row_number() == 1, adj.r.squared1, within.r.squared1),
      rr =  within.r.squared,
      rr1 = within.r.squared1) |> #ifelse(is.na(r.squared1), r.squared1, r.squared)) |>
    # glimpse()
    ungroup() |> 
    select(var, model, contains("rr"), contains("nvar")) |>
    mutate(across(where(is.numeric), ~ifelse(is.na(.), 0, .))) |> 
    mutate(contrib = rr - rr1,
           contrib = ifelse(model == "(1) Pooled" & var == "All variables", 
                            rr, contrib),
           n_var = number(nvar - nvar1)) |> 
    select(-contains("rr"), -nvar, -nvar1) |> 
    pivot_wider(names_from = model, values_from = contrib) 
  
}